@@ -38,13 +38,21 @@
 #include "pulseaudio-volume.h"
 
 
-static void                 pulseaudio_volume_finalize           (GObject            *object);
-static void                 pulseaudio_volume_connect            (PulseaudioVolume   *volume);
-static gdouble              pulseaudio_volume_v2d                (PulseaudioVolume   *volume,
-                                                                  pa_volume_t         vol);
-static pa_volume_t          pulseaudio_volume_d2v                (PulseaudioVolume   *volume,
-                                                                  gdouble             vol);
-static gboolean             pulseaudio_volume_reconnect_timeout  (gpointer            userdata);
+static void                 pulseaudio_volume_finalize           (GObject              *object);
+static void                 pulseaudio_volume_connect            (PulseaudioVolume     *volume);
+static gdouble              pulseaudio_volume_v2d                (PulseaudioVolume     *volume,
+                                                                  pa_volume_t           vol);
+static pa_volume_t          pulseaudio_volume_d2v                (PulseaudioVolume     *volume,
+                                                                  gdouble               vol);
+static gboolean             pulseaudio_volume_reconnect_timeout  (gpointer              userdata);
+static void                 pulseaudio_volume_get_sink_list_cb   (pa_context           *context,
+                                                                  const pa_sink_info   *i,
+                                                                  int                   eol,
+                                                                  void                 *userdata);
+static void                 pulseaudio_volume_get_source_list_cb (pa_context           *context,
+                                                                  const pa_source_info *i,
+                                                                  int                   eol,
+                                                                  void                 *userdata);
 
 
 struct _PulseaudioVolume
@@ -64,6 +72,16 @@ struct _PulseaudioVolume
   gboolean              muted_mic;
 
   guint                 reconnect_timer_id;
+
+  /* Device management */
+  GHashTable           *sinks;
+  GHashTable           *sources;
+
+  guint                 sink_index;
+  guint                 source_index;
+
+  gchar                *default_sink_name;
+  gchar                *default_source_name;
 };
 
 struct _PulseaudioVolumeClass
@@ -125,8 +143,14 @@ pulseaudio_volume_init (PulseaudioVolume *volume)
   volume->muted_mic = FALSE;
   volume->reconnect_timer_id = 0;
 
+  volume->default_sink_name = NULL;
+  volume->default_source_name = NULL;
+
   volume->pa_mainloop = pa_glib_mainloop_new (NULL);
 
+  volume->sinks = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
+  volume->sources = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
+
   pulseaudio_volume_connect (volume);
 }
 
@@ -139,6 +163,15 @@ pulseaudio_volume_finalize (GObject *object)
 
   volume->config = NULL;
 
+  if (volume->default_sink_name)
+    g_free (volume->default_sink_name);
+
+  if (volume->default_source_name)
+    g_free (volume->default_source_name);
+
+  g_hash_table_destroy (volume->sinks);
+  g_hash_table_destroy (volume->sources);
+
   pa_glib_mainloop_free (volume->pa_mainloop);
 
   (*G_OBJECT_CLASS (pulseaudio_volume_parent_class)->finalize) (object);
@@ -161,6 +194,8 @@ pulseaudio_volume_sink_info_cb (pa_context         *context,
   if (i == NULL) return;
   pulseaudio_debug ("sink info: %s, %s", i->name, i->description);
 
+  volume->sink_index = (guint)i->index;
+
   muted = !!(i->mute);
   vol = pulseaudio_volume_v2d (volume, i->volume.values[0]);
 
@@ -196,6 +231,8 @@ pulseaudio_volume_source_info_cb (pa_context           *context,
   if (i == NULL) return;
   pulseaudio_debug ("source info: %s, %s", i->name, i->description);
 
+  volume->source_index = (guint)i->index;
+
   muted_mic = !!(i->mute);
   vol_mic = pulseaudio_volume_v2d (volume, i->volume.values[0]);
 
@@ -240,6 +277,12 @@ pulseaudio_volume_sink_source_check (PulseaudioVolume *volume,
   g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
 
   pa_context_get_server_info (context, pulseaudio_volume_server_info_cb, volume);
+
+  g_hash_table_remove_all (volume->sinks);
+  g_hash_table_remove_all (volume->sources);
+
+  pa_context_get_sink_info_list (volume->pa_context, pulseaudio_volume_get_sink_list_cb, volume);
+  pa_context_get_source_info_list (volume->pa_context, pulseaudio_volume_get_source_list_cb, volume);
 }
 
 
@@ -260,12 +303,18 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
       pulseaudio_debug ("received sink event");
       break;
 
+    case PA_SUBSCRIPTION_EVENT_SINK_INPUT :
+      pulseaudio_volume_sink_source_check (volume, context);
+      pulseaudio_debug ("received sink output event");
+      break;
+
     case PA_SUBSCRIPTION_EVENT_SOURCE        :
       pulseaudio_volume_sink_source_check (volume, context);
       pulseaudio_debug ("received source event");
       break;
 
     case PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT :
+      pulseaudio_volume_sink_source_check (volume, context);
       pulseaudio_debug ("received source output event");
       break;
 
@@ -283,6 +332,72 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
 
 
 
+
+
+static void
+pulseaudio_volume_get_sink_list_cb (pa_context         *context,
+                                    const pa_sink_info *i,
+                                    int                 eol,
+                                    void               *userdata)
+{
+  gchar *name = NULL;
+  gchar *description = NULL;
+
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  if (eol > 0) return;
+
+  name = g_strdup (i->name);
+  description = g_strdup (i->description);
+
+  g_hash_table_insert (volume->sinks, name, description);
+}
+
+
+
+static void
+pulseaudio_volume_get_source_list_cb (pa_context           *context,
+                                      const pa_source_info *i,
+                                      int                   eol,
+                                      void                 *userdata)
+{
+  gchar *name = NULL;
+  gchar *description = NULL;
+
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  if (eol > 0) return;
+
+  /* Ignore sink monitors, not relevant for users */
+  if (i->monitor_of_sink != PA_INVALID_INDEX) return;
+
+  name = g_strdup (i->name);
+  description = g_strdup (i->description);
+
+  g_hash_table_insert (volume->sources, name, description);
+}
+
+
+
+static void
+pulseaudio_volume_get_server_info_cb (pa_context           *context,
+                                      const pa_server_info *i,
+                                      void                 *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  g_free (volume->default_sink_name);
+  g_free (volume->default_source_name);
+
+  volume->default_sink_name = g_strdup (i->default_sink_name);
+  volume->default_source_name = g_strdup (i->default_source_name);
+}
+
+
+
 static void
 pulseaudio_volume_context_state_cb (pa_context *context,
                                     void       *userdata)
@@ -292,15 +407,19 @@ pulseaudio_volume_context_state_cb (pa_context *context,
   switch (pa_context_get_state (context))
     {
     case PA_CONTEXT_READY        :
-      pa_context_subscribe (context, PA_SUBSCRIPTION_MASK_SINK | PA_SUBSCRIPTION_MASK_SOURCE | PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT | PA_SUBSCRIPTION_MASK_SERVER , NULL, NULL);
+      pa_context_subscribe (context, PA_SUBSCRIPTION_MASK_SINK | PA_SUBSCRIPTION_MASK_SINK_INPUT | PA_SUBSCRIPTION_MASK_SOURCE | PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT | PA_SUBSCRIPTION_MASK_SERVER , NULL, NULL);
       pa_context_set_subscribe_callback (context, pulseaudio_volume_subscribe_cb, volume);
 
       pulseaudio_debug ("PulseAudio connection established");
       volume->connected = TRUE;
       // Check current sink and source volume manually. PA sink events usually not emitted.
       pulseaudio_volume_sink_source_check (volume, context);
+
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+
+      pa_context_get_server_info (volume->pa_context, pulseaudio_volume_get_server_info_cb, volume);
+
       break;
 
     case PA_CONTEXT_FAILED       :
@@ -312,8 +431,13 @@ pulseaudio_volume_context_state_cb (pa_context *context,
       volume->muted = FALSE;
       volume->volume_mic = 0.0;
       volume->muted_mic = FALSE;
+
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+
+      g_hash_table_remove_all (volume->sinks);
+      g_hash_table_remove_all (volume->sources);
+
       if (volume->reconnect_timer_id == 0)
         volume->reconnect_timer_id = g_timeout_add_seconds
           (5, pulseaudio_volume_reconnect_timeout, volume);
@@ -459,22 +583,6 @@ pulseaudio_volume_sink_volume_changed (pa_context *context,
 
 
 
-/* mute setting callbacks */
-/* pa_sink_info_cb_t */
-static void
-pulseaudio_volume_set_muted_cb1 (pa_context         *context,
-                                 const pa_sink_info *i,
-                                 int                 eol,
-                                 void               *userdata)
-{
-  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
-
-  pa_context_set_sink_mute_by_index (context, i->index, volume->muted, pulseaudio_volume_sink_volume_changed, volume);
-}
-
-
-
 void
 pulseaudio_volume_set_muted (PulseaudioVolume *volume,
                              gboolean          muted)
@@ -486,7 +594,7 @@ pulseaudio_volume_set_muted (PulseaudioVolume *volume,
   if (volume->muted != muted)
     {
       volume->muted = muted;
-      pa_context_get_sink_info_list (volume->pa_context, pulseaudio_volume_set_muted_cb1, volume);
+      pa_context_set_sink_mute_by_index (volume->pa_context, volume->sink_index, volume->muted, pulseaudio_volume_sink_volume_changed, volume);
     }
 }
 
@@ -528,22 +636,6 @@ pulseaudio_volume_source_volume_changed (pa_context *context,
 
 
 
-/* mute setting callbacks */
-/* pa_source_info_cb_t */
-static void
-pulseaudio_volume_set_muted_mic_cb1 (pa_context           *context,
-                                     const pa_source_info *i,
-                                     int                   eol,
-                                     void                 *userdata)
-{
-  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
-
-  pa_context_set_source_mute_by_index (context, i->index, volume->muted_mic, pulseaudio_volume_source_volume_changed, volume);
-}
-
-
-
 void
 pulseaudio_volume_set_muted_mic (PulseaudioVolume *volume,
                                  gboolean          muted_mic)
@@ -555,7 +647,7 @@ pulseaudio_volume_set_muted_mic (PulseaudioVolume *volume,
   if (volume->muted_mic != muted_mic)
     {
       volume->muted_mic = muted_mic;
-      pa_context_get_source_info_list (volume->pa_context, pulseaudio_volume_set_muted_mic_cb1, volume);
+      pa_context_set_source_mute_by_index (volume->pa_context, volume->source_index, volume->muted_mic, pulseaudio_volume_source_volume_changed, volume);
     }
 }
 
@@ -699,6 +791,167 @@ pulseaudio_volume_set_volume_mic (PulseaudioVolume *volume,
 
 
 
+static gint
+sort_device_list (gchar *a,
+                  gchar *b,
+                  void  *hash_table)
+{
+  GHashTable *table = (GHashTable *)hash_table;
+  gchar      *a_val = (gchar *) g_hash_table_lookup (table, a);
+  gchar      *b_val = (gchar *) g_hash_table_lookup (table, b);
+  return g_strcmp0 (a_val, b_val);
+}
+
+
+
+GList *
+pulseaudio_volume_get_output_list (PulseaudioVolume *volume)
+{
+  GList *list;
+  GList *sorted;
+
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
+
+  list = g_hash_table_get_keys (volume->sinks);
+  sorted = g_list_sort_with_data (list, (GCompareDataFunc) sort_device_list, volume->sinks);
+
+  return sorted;
+}
+
+
+
+gchar *
+pulseaudio_volume_get_output_by_name (PulseaudioVolume *volume,
+                                      gchar            *name)
+{
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
+  return (gchar *) g_hash_table_lookup (volume->sinks, name);
+}
+
+
+
+GList *
+pulseaudio_volume_get_input_list (PulseaudioVolume *volume)
+{
+  GList *list;
+  GList *sorted;
+
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
+
+  list = g_hash_table_get_keys (volume->sources);
+  sorted = g_list_sort_with_data (list, (GCompareDataFunc) sort_device_list, volume->sources);
+
+  return sorted;
+}
+
+
+
+gchar *
+pulseaudio_volume_get_input_by_name (PulseaudioVolume *volume,
+                                     gchar            *name)
+{
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (volume));
+  return (gchar *) g_hash_table_lookup (volume->sources, name);
+}
+
+
+
+const gchar *
+pulseaudio_volume_get_default_output (PulseaudioVolume *volume)
+{
+  return volume->default_sink_name;
+}
+
+
+
+const gchar *
+pulseaudio_volume_get_default_input (PulseaudioVolume *volume)
+{
+  return volume->default_source_name;
+}
+
+
+
+static void
+pulseaudio_volume_default_sink_changed_info_cb (pa_context         *context,
+                                                const pa_sink_info *i,
+                                                int                 eol,
+                                                void               *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  pa_context_move_sink_input_by_index (context, volume->sink_index, i->index, NULL, NULL);
+  volume->sink_index = (guint)i->index;
+}
+
+
+
+static void
+pulseaudio_volume_default_sink_changed (pa_context *context,
+                                        int         success,
+                                        void       *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+
+  if (success)
+    pa_context_get_sink_info_by_name (volume->pa_context, volume->default_sink_name, pulseaudio_volume_default_sink_changed_info_cb, volume);
+}
+
+
+
+void
+pulseaudio_volume_set_default_output (PulseaudioVolume *volume,
+                                      gchar            *name)
+{
+  g_free (volume->default_sink_name);
+  volume->default_sink_name = g_strdup (name);
+
+  pa_context_set_default_sink (volume->pa_context, name, pulseaudio_volume_default_sink_changed, volume);
+}
+
+
+
+static void
+pulseaudio_volume_default_source_changed_info_cb (pa_context         *context,
+                                                  const pa_source_info *i,
+                                                  int                 eol,
+                                                  void               *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  if (i == NULL) return;
+
+  pa_context_move_source_output_by_index (context, volume->source_index, i->index, NULL, NULL);
+  volume->source_index = (guint)i->index;
+}
+
+
+
+static void
+pulseaudio_volume_default_source_changed (pa_context *context,
+                                          int         success,
+                                          void       *userdata)
+{
+  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+
+  if (success)
+    pa_context_get_source_info_by_name (volume->pa_context, volume->default_source_name, pulseaudio_volume_default_source_changed_info_cb, volume);
+}
+
+
+
+void
+pulseaudio_volume_set_default_input (PulseaudioVolume *volume,
+                                     gchar            *name)
+{
+  g_free (volume->default_source_name);
+  volume->default_source_name = g_strdup (name);
+
+  pa_context_set_default_source (volume->pa_context, name, pulseaudio_volume_default_source_changed, volume);
+}
+
+
+
 PulseaudioVolume *
 pulseaudio_volume_new (PulseaudioConfig *config)
 {
